--[[
--	Xparky is a rewrite of FuXPFu to use Ace3 and deFu it
--	Mouse frame selection shamelessly stolen from Dash (Kyhax)
--]]

Xparky = LibStub("AceAddon-3.0"):NewAddon("Xparky", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Xparky")
local reg = LibStub("AceConfigRegistry-3.0")
local dialog = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Xparky", "Xparky")
local _G = getfenv(0)

--http://wiki.github.com/tekkub/libdatabroker-1-1/api
local ldb = LibStub:GetLibrary("LibDataBroker-1.1");
local dataobj = ldb:NewDataObject("Xparky",
	{
	type = "launcher",
	label = "Xparky",
	OnClick = function(clickedFrame, button) InterfaceOptionsFrame_OpenToCategory("Xparky") end
	})


local options = {}
local db

local bars = {}
local barsindex = {}

local Shadow, Anchor

local Strata = { "BACKGROUND", "LOW", "MEDIUM", "HIGH", "DIALOG", "FULLSCREEN", "TOOLTIP" }

local default =
{
	profile =
	{
		bars = {},
		ScreenWidth = 100,
		Thickness = 4,
		Attach = "bottom",
		Inside = false,
		ConnectedFrame = "UIParent", --  defaults to off the bottom of game screen
		xOffset = 0,
		yOffset = 0,
		Strata = 5,
		MouseHide = false
	}
}

--[[ Local helper functions --]]
--

local function SetColour(aFrame, colours, texture)
	if colours then
		texture:SetVertexColor(colours.Red, colours.Green, colours.Blue, colours.Alpha)
		if aFrame.Spark then
			aFrame.Spark:SetVertexColor(colours.Red, colours.Green, colours.Blue, colours.Alpha)
			aFrame.Spark2:SetVertexColor(colours.Red, colours.Green, colours.Blue, colours.Alpha)
		end
	end
end

local function getWidth(aFrame)
	if db.Attach == "top" or db.Attach == "bottom" then
		return aFrame:GetWidth()
	else
		return aFrame:GetHeight()
	end
end

local function setWidth(aFrame, Size)
	if db.Attach == "top" or db.Attach == "bottom" then
		aFrame:SetWidth(Size)
	else
		aFrame:SetHeight(Size)
	end
end

local function getHeight(aFrame)
	if db.Attach == "top" or db.Attach == "bottom" then
		return aFrame:GetHeight()
	else
		return aFrame:GetWidth()
	end
end

local function setHeight(aFrame, Size)
	if db.Attach == "top" or db.Attach == "bottom" then
		aFrame:SetHeight(Size)
	else
		aFrame:SetWidth(Size)
	end
end


local function CreateBar(frame, colours, sparkAlpha)
	local tex = frame:CreateTexture(frame.Name .. "Texture", "PARENT")
	tex:SetTexture(frame.TextureFile)
	tex:ClearAllPoints()
	tex:SetAllPoints(frame)
	tex:Show()
	frame.Texture = tex
	setHeight(frame, db.Thickness)
	if sparkAlpha and sparkAlpha > 0 then
		
		local spark = frame:CreateTexture(frame.Name .. "Spark", "PARENT")
		spark:SetTexture(frame.Spark1File)
		setHeight(spark, db.Thickness * 5)
		spark:SetBlendMode("ADD")
		spark:SetParent(frame)
		spark:SetAlpha(sparkAlpha)
		frame.Spark = spark

		local spark2 = frame:CreateTexture(frame.Name .. "Spark2", "PARENT")
		spark2:SetTexture(frame.Spark2File)
		setHeight(spark2, db.Thickness * 5)
		spark2:SetBlendMode("ADD")
		spark2:SetParent(frame)
		spark2:SetAlpha(sparkAlpha)
		frame.Spark2 = spark2
	end
	SetColour(frame, colours, tex)
	frame:ClearAllPoints()
	setWidth(frame, 100)
	return frame
end


local function GenerateBar(BarName, colours, sparkAlpha)
	local frame = CreateFrame("Frame", BarName .. "Xparky", Anchor)
	frame.Name = BarName
	frame.TextureFile = "Interface\\AddOns\\Xparky\\Textures\\texture.tga"
	frame.Spark1File =  "Interface\\AddOns\\Xparky\\Textures\\glow.tga"
	frame.Spark2File =  "Interface\\AddOns\\Xparky\\Textures\\glow2.tga"
	return CreateBar(frame, colours, sparkAlpha)
end


local mouser = CreateFrame("Frame")
mouser.tooltip = _G.GameTooltip
mouser.setCursor = _G.SetCursor

function mouser:OnUpdate(elap)
    if IsMouseButtonDown("RightButton") then
        return self:Stop()
    end

    local frame = GetMouseFocus()
    local name = frame and frame:GetName() or tostring(frame)
    
    SetCursor("CAST_CURSOR")
    if not frame then return end
    self.tooltip:SetOwner(frame, "ANCHOR_BOTTOMLEFT")
    self.tooltip:SetText(name, 1.0, 0.82, 0)
    self.tooltip:Show()
    
    if IsMouseButtonDown("LeftButton") then
        self:Stop()
        if not type(frame.GetName) == 'function' or not frame:GetName() then
            Xparky:Print(L["This frame has no global name, and cannot be added via the mouse"])
        else
        	db.ConnectedFrame = name
        	Xparky:AttachBar()
        	reg:NotifyChange("Xparky")
        end
    end
end

function mouser:Start()
    self:SetScript("OnUpdate", self.OnUpdate)
end

function mouser:Stop()
    self.tooltip:Hide()
    self:SetScript("OnUpdate", nil)
end
hooksecurefunc(_G.GameMenuFrame, "Show", function() mouser:Stop() end)


options.type = "group"
options.name  = "Xparky"
options.get  = function( k )  return db[k.arg] end
options.set  = function( k, v ) db[k.arg] = v; Xparky:UpdateBars(k.arg) end
options.args = {}

options.args.bars = {
	type = "group",
	name = L["Bars"],
	desc = L["Bar Modifications"],
	order = 1,
	args = {
		thick = {
			order = 2,
			name = L["Bar Thickness"],
			desc = L["How thick the bars are"],
			type = "range",
			min = 1.5, max = 8, step = 0.1,
			arg = "Thickness"
		},
		hide = {
			order = 1,
			name = L["Hide Bars"],
			desc = L["Hide the bars til you mouseover them"],
			type = "toggle",
			arg = "MouseHide",
		},
	},
}

options.args.framelink = {
	type = "group",
	name = L["Frame Link"],
	order = 2,
	args = {
		attach = {
			order = 1,
			type = "execute",
			name = L["Hook to frame"],
			desc = L["Click here to activate the frame selector"],
			func = function() mouser:Start() end
		},
		space = {
			order = 2,
			name = "",
			desc = "",
			type = "description"
		},
		attached = {
			order = 3,
			type = "input",
			name = L["Frame Connected to"],
			desc = L["The name of the frame to connect to"],
			arg = "ConnectedFrame",
			set = function(k,v) db.ConnectedFrame = v; db.Detached = false; Xparky:AttachBar() end
		},
		attachto = {
			order = 4,
			name = L["Attach to:"],
			desc = L["Which side to attach to"],
			type = "select",
			values = { top = L["Top"], bottom = L["Bottom"], left = L["Left"], right = L["Right"] },
			arg = "Attach"
		},
		insideframe = {
			order = 5,
			name = L["Inside Frame?"],
			desc = L["Attach to the inside of the frame"],
			type = "toggle",
			arg = "Inside"
		},
		spacer = {
			order = 6,
			name = L["Bar Strata"],
			desc = L["Set the Bar Strata so it appears above or below other frames"],
			type = "select",
			values = Strata,
			arg = "Strata"
		},
		xoffset = {
			order = 7,
			name = L["X offset"],
			desc = L["How far on the X axis to offset the bars"],
			type = "input",
			arg = "xOffset"
		},
		yoffset = {
			order = 8,
			name = L["Y offset"],
			desc = L["How far on the Y axis to offset the bars"],
			type = "input",
			arg = "yOffset"
		},
	},
}

options.args.help = {
	type = "group",
	order = 100,
	name = L["Documentation"],
	args = {
		desc = {
			type = "group",
			name = L["Description"],
			args = {
				text = {
					type = "description",
					name = L["DESCRIPTION"],
					order = 1
				},
			},
		},
		faq = { 
			type = "group",
			name = L["FAQ"],
			args = {
				text = {
					type = "description",
					name = L["FAQ_TEXT"],
					order = 2
				},
			},
		},
		about = {
			type = "group",
			name = L["About"],
			args = {
				text = {
					type = "description",
					name = L["ADDON_INFO"],
					order = 3
				}
			}
		}
	}
}


local function MouseOver()
	if db.MouseHide then
		Xparky:ConnectBars()
	end
end

local function HideBars()
	for _, aBar in pairs(bars) do
		aBar:hideBars()
	end
end


local function MouseOut()
	if db.MouseHide then
		HideBars()
	end
end

local function SetStrata()
	for _, aBar in pairs(bars) do
		aBar:setStrata(db.Strata)
	end
end

function Xparky:OnInitialize()
	self:RegisterEvent("PLAYER_LOGIN") -- Only fires once
	self:RegisterEvent("PLAYER_ENTERING_WORLD") -- Initial load, reload, change instance, repspawn (any load screen)
	barsindex = { "XP", "Reputation", "Shadow"}

	bars.XP = barXP:new("XP")
	bars.Reputation = barRep:new("Reputation")
	bars.Shadow = barShadow:new("Reputation")
	
	bars.XP:init("XP", 1)
	bars.Reputation:init("Reputation", 1)
	bars.Shadow:init("Shadow", 5)

	local order = 0
	for _, v in ipairs(barsindex) do
		bars[v]:setOptions(order)
		bars[v]:setDefaults()
		
		order = order + 1
	end
	
end

function Xparky:PLAYER_LOGIN()
	Xparky.db = LibStub("AceDB-3.0"):New("XparkyDB", default, "Default")
	db = Xparky.db.profile
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Xparky.db)
	reg:RegisterOptionsTable("Xparky", options)
	self:RegisterChatCommand("xparky", function() InterfaceOptionsFrame_OpenToCategory("Xparky") end)
	Xparky:InitializeBars()
	Xparky:ConnectBars()
	Xparky:AttachBar()
	Xparky:InitialiseEvents()

	if db.MouseHide then 
		Anchor:EnableMouse(true)
	    Anchor:SetScript("OnEnter",MouseOver)
		Anchor:SetScript("OnLeave",MouseOut) 
		HideBars()
	end

end

function Xparky:PLAYER_ENTERING_WORLD()
	self:UpdateBars()
end


function Xparky:InitializeBars()
	Anchor = CreateFrame("Frame", "XparkyXPAnchor", UIParent)
	Anchor:SetWidth(1)
	Anchor:SetHeight(1)
	Anchor:Show()
	for _, aBar in pairs(bars) do
		aBar:generateFrames()
	end
end

function Xparky:ConnectBars()
	local Base = Anchor
	local TabA, SlotB, TabC, SlotD
	local tlx, tly, trx, try, blx, bly, brx, bry,stlx, stly, strx, stry, sblx, sbly, sbrx, sbry

	if (db.Attach == "bottom" and not db.Inside) or (db.Attach == "top" and db.Inside) then
		TabA = "TOPLEFT"
		SlotB = "BOTTOMLEFT"
		tlx, tly, trx, try, blx, bly, brx, bry = 1, 0, 1, 1, 0, 0, 0, 1
		stlx, stly, strx, stry, sblx, sbly, sbrx, sbry = 0, 1, 0, 0, 1, 1, 1, 0
	end
	
	if (db.Attach == "top" and not db.Inside) or (db.Attach == "bottom" and db.Inside) then
		TabA = "BOTTOMLEFT"
		SlotB = "TOPLEFT"
		tlx, tly, trx, try, blx, bly, brx, bry = 0, 1, 0, 0, 1, 1, 1, 0
		stlx, stly, strx, stry, sblx, sbly, sbrx, sbry = 0, 1, 0, 0, 1, 1, 1, 0
	end

	if (db.Attach == "left" and not db.Inside) or (db.Attach == "right" and db.Inside) then
		TabA = "TOPRIGHT"
		SlotB = "TOPLEFT"
		tlx, tly, trx, try, blx, bly, brx, bry = 1, 1, 0, 1, 1, 0, 0, 0
		stlx, stly, strx, stry, sblx, sbly, sbrx, sbry = 0, 0, 1, 0, 0, 1, 1, 1
	end

	if (db.Attach == "right" and not db.Inside) or (db.Attach == "left" and db.Inside) then
		TabA = "TOPLEFT"
		SlotB = "TOPRIGHT"
		tlx, tly, trx, try, blx, bly, brx, bry = 0, 0, 1, 0, 0, 1, 1, 1
		stlx, stly, strx, stry, sblx, sbly, sbrx, sbry = 0, 0, 1, 0, 0, 1, 1, 1
	end

	if db.Attach == "bottom" or db.Attach == "top" then
		TabC = "LEFT"
		SlotD = "RIGHT"
	end

	if db.Attach == "left" or db.Attach == "right" then
		TabC = "TOP"
		SlotD = "BOTTOM"
	end

	local barEnd, x, y = "", 0, 0
	if db.Attach == "top" or db.Attach == "bottom" then
		barEnd = "RIGHT"
		x = 10
		y = 0
	end

	if db.Attach == "left" or db.Attach == "right" then
		barEnd = "BOTTOM"
		x = 0
		y = -10
	end

	for _, aBar in pairs(bars) do
		aBar:hideBars()
	end
	
	for _, v in ipairs(barsindex) do
		if db.bars[bars[v].name].show then
			Base = bars[v]:connectBar ( Base, TabA, SlotB, TabC, SlotD, tlx, tly, trx, try,blx, bly, brx, bry, stlx, stly, strx, stry, sblx, sbly, sbrx, sbry, barEnd, x, y )
		end
	end
	
	SetStrata()
end
do

	local timeout = 0

	function Xparky:AttachBar(Bar)
		local Foundation = db.ConnectedFrame and getglobal(db.ConnectedFrame) or nil
		if Foundation then
			Anchor:ClearAllPoints()

			if db.Attach == "bottom" then
				if db.Inside then
					Anchor:SetPoint("BOTTOMLEFT", Foundation, "BOTTOMLEFT", db.xOffset, db.yOffset )
				else
					Anchor:SetPoint("TOPLEFT", Foundation, "BOTTOMLEFT", db.xOffset, db.yOffset )
				end
			elseif db.Attach == "top" then
				if db.Inside then
					Anchor:SetPoint("TOPLEFT", Foundation, "TOPLEFT", db.xOffset, db.yOffset )
				else
					Anchor:SetPoint("BOTTOMLEFT", Foundation, "TOPLEFT", db.xOffset, db.yOffset )
				end
			elseif db.Attach == "left" then
				if db.Inside then
					Anchor:SetPoint("TOPLEFT", Foundation, "TOPLEFT", db.xOffset, db.yOffset )
				else
					Anchor:SetPoint("TOPRIGHT", Foundation, "TOPLEFT", db.xOffset, db.yOffset )
				end
			elseif db.Attach == "right" then
				if db.Inside then
					Anchor:SetPoint("TOPRIGHT", Foundation, "TOPRIGHT", db.xOffset, db.yOffset )
				else
					Anchor:SetPoint("TOPLEFT", Foundation, "TOPRIGHT", db.xOffset, db.yOffset )
				end
			end

			Anchor:SetParent(Foundation)
			self:ConnectBars()
			self:UpdateBars()
		else
			if timeout > 5 then
				self:Print(L["Cannot find frame specified"])
				timeout = 0
				return
			end
			self:ScheduleTimer("AttachBar", 1, self)
			timeout = timeout + 1
		end 
	end
 
end

function Xparky:InitialiseEvents()
	for _, aBar in pairs(bars) do
		aBar:initialiseEvents()
	end
end

function Xparky:UpdateBars(dimensions)
	local total =  getWidth(Anchor:GetParent())
	local anchor

	anchor = 0
	
	for _, aBar in pairs(bars) do
	
		if db.bars[aBar.name].show then
			anchor = anchor + aBar:updateBar(total)
		end
		
	end

	setWidth(Anchor, total)
	setHeight(Anchor, anchor)

	if type(dimensions) == "string" then
		if dimensions == "Thickness" then
			for _, aBar in pairs(bars) do
				aBar:setHeight(db.Thickness)
			end
		elseif string.match(dimensions, "Offset") then
			self:AttachBar()
		elseif string.match(dimensions, "Mouse") then
			Anchor:SetScript("OnEnter", MouseOver)
			Anchor:SetScript("OnLeave", MouseOut) 
			Anchor:EnableMouse(true)
			if not db.MouseHide then
				Xparky:ConnectBars()
			else
				HideBars()
			end
		elseif string.match(dimensions,"ToGo") then
			self:AttachBar()
		elseif dimensions == "Attach" then
			self:UpdateBars("Thickness")
			self:AttachBar()
		elseif dimensions == "Inside" then
			self:AttachBar()
		elseif dimensions == "Strata" then
			SetStrata()
		end
	end
end



--
-- Classes
--


function CreateClass(base_class)

	local new_class = {}
	
	if base_class ~= nil then
		new_class = base_class.new()
	end
	
	local class_mt = { __index = new_class }

	function new_class.new()
		local newinst = {}
		setmetatable( newinst, class_mt )
		return newinst
	end

	return new_class
end


--
-- class bar
--
bar = CreateClass()

function bar:init(aName, aHeight)
	self.name = aName
	self.height = aHeight
end


function bar:connectBar
(
	Base, TabA, SlotB, TabC, SlotD,
	tlx, tly, trx, try,
	blx, bly, brx, bry,
	stlx, stly, strx, stry,
	sblx, sbly, sbrx, sbry,
	barEnd, x, y
)
	self.fillFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)
	self.fillFrame.Spark:SetTexCoord(stlx, stly, strx, stry, sblx, sbly, sbrx, sbry)
	self.fillFrame.Spark2:SetTexCoord(stlx, stly, strx, stry, sblx, sbly, sbrx, sbry)
	self.emptyFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)

	self.fillFrame:ClearAllPoints()
	self.fillFrame:SetPoint(TabA, Base, Base == Anchor and TabA or SlotB)
	self.fillFrame:SetFrameLevel( self.emptyFrame:GetFrameLevel() + 1)
	self.fillFrame.Spark:ClearAllPoints()
	self.fillFrame.Spark:SetPoint(barEnd, self.fillFrame, barEnd, x, y)
	self.fillFrame.Spark:SetParent(self.fillFrame)
	self.fillFrame.Spark:SetAlpha(db.bars[self.name].sparkAlpha)
	self.fillFrame.Spark2:ClearAllPoints()
	self.fillFrame.Spark2:SetPoint(barEnd, self.fillFrame, barEnd, x, y)
	self.fillFrame.Spark2:SetParent(self.fillFrame)
	self.fillFrame.Spark2:SetAlpha(db.bars[self.name].sparkAlpha)
	self.emptyFrame:ClearAllPoints()
	self.emptyFrame:SetPoint(TabC, self.fillFrame, SlotD)
	self:showBars()
	return self.fillFrame

end


function bar:hideBars()
	self.fillFrame:Hide()
	self.emptyFrame:Hide()
end


function bar:showBars()
	self.fillFrame:Show()
	self.emptyFrame:Show()
end


function bar:updateBar(width)
	local split = self:getSplit()
			
	setWidth( self.fillFrame, split * width + 0.001)
	setWidth( self.emptyFrame, (1 - split) * width + 0.001)
	
	setWidth( self.fillFrame.Spark, getWidth(self.fillFrame) < 20 and getWidth(self.fillFrame) * 5 or 128)
	setWidth( self.fillFrame.Spark2, getWidth(self.fillFrame) < 20 and getWidth(self.fillFrame) * 5 or 128)
	
	return self.height
end


function bar:generateFrames()
	self.fillFrame = GenerateBar(
		self.name.."Bar",
		db.bars[self.name].colour,
		db.bars[self.name].sparkAlpha
		)
	self.emptyFrame = GenerateBar(
		"No"..self.name.."Bar",
		db.bars[self.name].emptyColour
		)
end


function bar:getSplit()
	return 1
end


function bar:setDefaults()
	default.profile.bars[self.name] =
	{
		name = self.name,
		colour = { Red = 1, Green = 1, Blue = 1, Alpha = 1},
		emptyColour = { Red = 0.2, Green = 0.2, Blue = 0.2, Alpha = 0.33},
		sparkAlpha = 0.5,
		show = true,
	}
end


function bar:initialiseEvents()

end


function bar:setHeight(aHeight)
	self.height = aHeight
	setHeight(self.fillFrame, self.height)
	setHeight(self.emptyFrame, self.height)
	setHeight(self.fillFrame.Spark, self.height * 8)
	setHeight(self.fillFrame.Spark2, self.height * 8)
end


function bar:setOptions(order)

	options.args.bars.args[self.name] =
	{
		type = "group",
		name = self.name,
		desc = "Settings for "..self.name.." bar",
		order = order,
		args =
		{
			show =
			{
				order = 1,
				name = "Show",
				desc = "Whether to show the bar or not",
				type = "toggle",
				get = function(info)
					return db.bars[info[#info-1]][info[#info]]
				end,
				set = function(info, value)
					db.bars[info[#info-1]][info[#info]] = value
					Xparky:ConnectBars()
					Xparky:UpdateBars()
				end
			},
			sparkAlpha =
			{
				order = 2,
				name = "Spark Intensity",
				desc = "How strong the bar's spark is",
				type = "range",
				min = 0.0, max = 1, step = 0.05,
				get = function(info)
					return db.bars[info[#info-1]][info[#info]]
				end,
				set = function(info, value)
					db.bars[info[#info-1]][info[#info]] = value
					
					bars[info[#info-1]].fillFrame.Spark:SetAlpha(value)
				end
			},
			colour = 
			{
				order = 3,
				name = "Bar Colour",
				desc = "Colour of the full bar",
				type = "color",
				hasAlpha = true,
				get = function(info)
					local t = db.bars[info[#info-1]][info[#info]]
						or { Red = 1, Green = 1, Blue = 1, Alpha = 1}
						
					return t.Red, t.Green, t.Blue, t.Alpha
				end,
				set = function(info, r, g, b, a)
					db.bars[info[#info-1]][info[#info]] =
					{
						Red = r,
						Green = g,
						Blue = b,
						Alpha = a
					}
					SetColour(
						bars[info[#info-1]].fillFrame,
						db.bars[info[#info-1]][info[#info]],
						bars[info[#info-1]].fillFrame.Texture)
				end,
			},
			emptyColour = 
			{
				order = 99,
				name = "Empty Bar Colour",
				desc = "Colour of the empty bar",
				type = "color",
				hasAlpha = true,
				get = function(info)
					local t = db.bars[info[#info-1]][info[#info]]
						or { Red = 1, Green = 1, Blue = 1, Alpha = 1}
						
					return t.Red, t.Green, t.Blue, t.Alpha
				end,
				set = function(info, r, g, b, a)
					db.bars[info[#info-1]][info[#info]] =
					{
						Red = r,
						Green = g,
						Blue = b,
						Alpha = a
					}
					SetColour(
						bars[info[#info-1]].emptyFrame,
						db.bars[info[#info-1]][info[#info]],
						bars[info[#info-1]].emptyFrame.Texture)
				end,
			},
		},
	}
end


function bar:setStrata(aStrata)
	self.fillFrame:SetFrameStrata(Strata[aStrata])
	self.emptyFrame:SetFrameStrata(Strata[aStrata])

end

--
-- class barXP
--
barXP = CreateClass(bar)


function barXP:setDefaults()
	bar.setDefaults(self)
	
	default.profile.bars[self.name].colour = { Red = 0, Green = 0, Blue = 1, Alpha = 1 }
	default.profile.bars[self.name].restedColour = { Red = 1, Green = 0.2, Blue = 1, Alpha = 1 }
end

function barXP:setOptions(order)
	bar.setOptions(self, order)

	options.args.bars.args[self.name].args.restedColour =
	{
		order = 4,
		name = "Rest Bar Colour",
		desc = "Colour of the rested bar",
		type = "color",
		hasAlpha = true,
		get = function(info)
			local t = db.bars[info[#info-1]][info[#info]]
				or { Red = 1, Green = 1, Blue = 1, Alpha = 1}
				
			return t.Red, t.Green, t.Blue, t.Alpha
		end,
		set = function(info, r, g, b, a)
			db.bars[info[#info-1]][info[#info]] =
			{
				Red = r,
				Green = g,
				Blue = b,
				Alpha = a
			}
			SetColour(
				bars[info[#info-1]].restedFrame,
				db.bars[info[#info-1]][info[#info]],
				bars[info[#info-1]].restedFrame.Texture)
		end,
	}
end


function barXP:getSplit()
	local currentXP, maxXP, restXP, remainXP, restSplit
		
	currentXP = UnitXP("player")
	maxXP = UnitXPMax("player")
	restXP = GetXPExhaustion() or 0
	remainXP = maxXP - (currentXP + restXP)
	if remainXP < 0 then
		remainXP = 0
	end
	
	if (restXP + currentXP)/maxXP > 1 then
		restSplit = 1 - currentXP/maxXP
	else
		restSplit = restXP/maxXP
	end
	
	return	currentXP/maxXP, restSplit
end


function barXP:hideBars()
	bar.hideBars(self)
	self.restedFrame:Hide()
end


function barXP:showBars()
	bar.showBars(self)
	self.restedFrame:Show()
end


function barXP:connectBar
(
	Base, TabA, SlotB, TabC, SlotD,
	tlx, tly, trx, try,
	blx, bly, brx, bry,
	stlx, stly, strx, stry,
	sblx, sbly, sbrx, sbry,
	barEnd, x, y
)
	self.fillFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)
	self.fillFrame.Spark:SetTexCoord(stlx, stly, strx, stry, sblx, sbly, sbrx, sbry)
	self.fillFrame.Spark2:SetTexCoord(stlx, stly, strx, stry, sblx, sbly, sbrx, sbry)
	self.fillFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)
	self.emptyFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)

	self.fillFrame:ClearAllPoints()
	self.fillFrame:SetPoint(TabA, Base, Base == Anchor and TabA or SlotB)
	self.fillFrame:SetFrameLevel( self.emptyFrame:GetFrameLevel() + 1)
	self.fillFrame.Spark:ClearAllPoints()
	self.fillFrame.Spark:SetPoint(barEnd, self.fillFrame, barEnd, x, y)
	self.fillFrame.Spark:SetParent(self.fillFrame)
	self.fillFrame.Spark:SetAlpha(db.bars[self.name].sparkAlpha)
	self.fillFrame.Spark2:ClearAllPoints()
	self.fillFrame.Spark2:SetPoint(barEnd, self.fillFrame, barEnd, x, y)
	self.fillFrame.Spark2:SetParent(self.fillFrame)
	self.fillFrame.Spark2:SetAlpha(db.bars[self.name].sparkAlpha)
	self.restedFrame:ClearAllPoints()
	self.restedFrame:SetPoint(TabC, self.fillFrame, SlotD)
	self.emptyFrame:ClearAllPoints()
	self.emptyFrame:SetPoint(TabC, self.restedFrame, SlotD)
	self:showBars()
	return self.fillFrame

end


function barXP:updateBar(width)
	local split1, split2 = self:getSplit()
			
	setWidth( self.fillFrame, split1 * width + 0.001)
	setWidth( self.restedFrame, split2 * width + 0.001)
	setWidth( self.emptyFrame, (1 - (split1 + split2)) * width + 0.001)
	
	setWidth( self.fillFrame.Spark, getWidth(self.fillFrame) < 20 and getWidth(self.fillFrame) * 5 or 128)
	setWidth( self.fillFrame.Spark2, getWidth(self.fillFrame) < 20 and getWidth(self.fillFrame) * 5 or 128)
	
	return self.height

end


function barXP:generateFrames()
	bar.generateFrames(self)
	self.restedFrame = GenerateBar(
		"Rested"..self.name.."Bar",
		db.bars[self.name].restedColour
		)
end


function barXP:initialiseEvents()
	Xparky:RegisterEvent("PLAYER_XP_UPDATE", "UpdateBars")
	Xparky:RegisterBucketEvent("UPDATE_EXHAUSTION", 60, "UpdateBars")
end


function barXP:setHeight(aHeight)
	bar.setHeight(self, aHeight)
	setHeight(self.restedFrame, db.Thickness)
end


function barXP:setStrata(aStrata)
	bar.setStrata(self, aStrata)
	self.restedFrame:SetFrameStrata(Strata[aStrata])
end

--
-- class barRep
--
barRep = CreateClass(bar)


function barRep:setDefaults()
	bar.setDefaults(self)
	
	default.profile.bars[self.name].colour = { Red = 0, Green = 1, Blue = 0, Alpha = 1 }
end


function barRep:getSplit()
	local repName, repLevel, minRep, maxRep, currentRep
		
	repName, repLevel, minRep, maxRep, currentRep = GetWatchedFactionInfo()
	
	-- 90001 seems to handle max rep differently, so assume max rep when no difference between max and min on the level
	if (maxRep - minRep) == 0 then
		return 100
	end
	
	return ( (currentRep - minRep) / (maxRep - minRep) )
end


function barRep:initialiseEvents()
	Xparky:RegisterBucketEvent("UPDATE_FACTION", 0.5, "UpdateBars")
end

--
-- class barShadow
--
barShadow = CreateClass(bar)


function barShadow:connectBar
(
	Base, TabA, SlotB, TabC, SlotD,
	tlx, tly, trx, try,
	blx, bly, brx, bry,
	_, _, _, _,
	_, _, _, _,
	_, _, _
)
	self.fillFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)
	self.fillFrame.Texture:SetTexCoord(tlx, tly, trx, try, blx, bly, brx, bry)

	self.fillFrame:ClearAllPoints()
	self.fillFrame:SetPoint(TabA, Base, Base == Anchor and TabA or SlotB)
	self:showBars()
	return self.fillFrame

end


function barShadow:hideBars()
	self.fillFrame:Hide()
end


function barShadow:showBars()
	self.fillFrame:Show()
end


function barShadow:updateBar(width)
	setWidth( self.fillFrame, width + 0.001)
	return self.height
end


function barShadow:setDefaults()
	default.profile.bars[self.name] =
	{
		name = self.name,
		colour = { Red = 0, Green = 0, Blue = 0, Alpha = 1},
		show = true,
	}
end


function barShadow:generateFrames()
	self.fillFrame = GenerateBar( self.name.."Bar", db.bars[self.name].colour)
	self.fillFrame.Texture:SetTexture("Interface\\AddOns\\Xparky\\Textures\\border.tga")
	self.fillFrame.Texture:SetVertexColor(
		db.bars[self.name].colour.Red,
		db.bars[self.name].colour.Green,
		db.bars[self.name].colour.Blue,
		db.bars[self.name].colour.Alpha
		)
	self.fillFrame.Texture:SetHeight(5)
end


function barShadow:setHeight(aHeight)
	self.height = aHeight
	setHeight(self.fillFrame, self.height)
end


function barShadow:setOptions(order)

	options.args.bars.args[self.name] =
	{
		type = "group",
		name = self.name,
		desc = "Settings for "..self.name.." bar",
		order = order,
		args =
		{
			show =
			{
				order = 1,
				name = "Show",
				desc = "Whether to show a shadow or not",
				type = "toggle",
				get = function(info)
					return db.bars[info[#info-1]][info[#info]]
				end,
				set = function(info, value)
					db.bars[info[#info-1]][info[#info]] = value
					Xparky:ConnectBars()
					Xparky:UpdateBars()
				end
			},
		},
	}
end


function barShadow:setStrata(aStrata)
	self.fillFrame:SetFrameStrata(Strata[aStrata])

end
